"""
Unit tests from properties.
"""
import unittest
from pyqcy import *


class Properties(unittest.TestCase):
    """Test cases for functionality related to properties."""

    def test_passing_property(self):
        subtraction_doesnt_break.test()

    def test_failing_property(self):
        self.assertRaises(AssertionError, failing.test)

    def test_parametrized_property(self):
        assert adding_to(5).test()


# Arbitraries

@arbitrary(int)
def two_digit_integers():
    return random.randint(10, 99)


# Test properties

@qc
def subtraction_doesnt_break(x=int, y=int):
    _ = x + y


@qc
def failing():
    assert False


@qc
def adding_to(x=0, y=int_(min=0, max=10)):
    assert x + y >= x



"""
Properties to be tested.
Also known as "tests".
"""
import inspect
import functools

from pyqcy.arbitraries import arbitrary, is_arbitrary, to_arbitrary
from pyqcy.statistics import Tag
from pyqcy.utils import optional_args


__all__ = ['qc']


DEFAULT_TEST_COUNT = 100


class Property(object):
    """A property that can be QuickChecked."""

    def __init__(self, args, kwargs, func):
        """Constructor. Callers should specify the function
        which encodes the testing property, and arbitrary values'
        generator for both positonal and keyword arguments.
        """
        self.args = [to_arbitrary(arg) if is_arbitrary(arg) else arg
                     for arg in args]
        self.kwargs = dict((k, to_arbitrary(v) if is_arbitrary else v)
                           for k, v in kwargs.iteritems())
        self.func = self.__coerce_to_generator_func(func)

    def __coerce_to_generator_func(self, func):
        """Ensures that given function is a generator function,
        i.e. a function that returns a generator.
        This way all properties can be tested the same way,
        regardless of whether they use `yield` internally or not.
        """
        if inspect.isgeneratorfunction(func):
            return func

        @functools.wraps(func)
        def generator_func(*args, **kwargs):
            func(*args, **kwargs)
            yield
        return generator_func

    def test(self, count=DEFAULT_TEST_COUNT):
        """Executes given number of tests for this property
        and gathers statistics about all test runs.
        Returns a list containing a set of "tags"
        for each test case that was executed.
        """
        if not (count > 0):
            raise ValueError("test count must be positive")
        return [self.test_one() for _ in xrange(count)]

    def test_one(self):
        """Executes a single test for this property."""
        args = self.__arbitrary_args()
        kwargs = self.__arbitrary_kwargs()
        coroutine = self.func(*args, **kwargs)
        return self.__execute_test(coroutine)

    def __arbitrary_args(self):
        """Returns a list of arbitrary values for
        positional arguments of the property function.
        """
        return [next(arg) if is_arbitrary(arg) else arg
                for arg in self.args]

    def __arbitrary_kwargs(self):
        """Returns a dictionary of arbitrary values
        for keyword arguments of the property function.
        """
        return dict((k, next(v) if is_arbitrary(v) else v)
                    for k, v in self.kwargs.iteritems())

    def __execute_test(self, coroutine):
        """Executes given test coroutine and returns results.
        The results is a set of "tags" that have been assigned to
        the test case by the property.
        """
        res = []
        try:
            while True:
                obj = next(coroutine)
                if obj is not None:
                    value = obj.value if isinstance(obj, Tag) else obj
                    res.append(value)
        except StopIteration:
            return frozenset(res)

    @property
    def parametrized(self):
        '''Checks whether the property is parametrized, i.e. has
        some arguments which are not generators of arbitrary values.
        '''
        args_are_arbitrary = all(map(is_arbitrary, self.args))
        kwargs_are_arbitrary = all(map(is_arbitrary, self.kwargs.itervalues()))
        return not (args_are_arbitrary and kwargs_are_arbitrary)

    def __call__(self, *args, **kwargs):
        """Calling the property object will create new one
        with some of the arguments bound to given values.
        In other words, it will perform currying on property's
        function.
        """
        @functools.wraps(self.func)
        def curried_func(*args_, **kwargs_):
            final_args = list(args) + list(args_ or [])
            final_kwargs = kwargs.copy()
            final_kwargs.update(kwargs_)
            return self.func(*final_args, **final_kwargs)

        curried_args = self.args[len(args):]
        curried_kwargs = dict((k, v) for (k, v) in self.kwargs.iteritems()
                               if k not in kwargs)
        return Property(curried_args, curried_kwargs, curried_func)


# The @qc decorator

@optional_args
class qc(object):
    """Decorator to be applied on functions that encode
    QuickCheck properties to be tested.
    """
    def __init__(self):
        pass  # argumentless, for now

    def __call__(self, func):
        """Applies the @qc decorator to given function,
        turning it into a testable property.
        """
        func_args, _, _, func_defaults = inspect.getargspec(func)
        func_args, func_defaults = func_args or [], func_defaults or []
        free_args_count = len(func_args) - len(func_defaults)
        if free_args_count > 0:
            raise TypeError("property has unbound variables: %s" %
                func_args[:free_args_count])
        return Property(args=func_defaults, kwargs={}, func=func)



"""
Unit tests for statistics.
"""
import unittest
from pyqcy import *


class Statistics(unittest.TestCase):
    """Test cases for statistics functionality."""

    def test_collect(self):
        results = sorting_short_lists.test()
        assert all(len(r) > 0 for r in results)

    def test_classify(self):
        results = sort_preserves_length.test()
        assert any(len(r) > 0 for r in results)


# Test properties

@qc
def sorting_short_lists(
    l=list_(float, min_length=1, max_length=6)
):
    yield collect(len(l))
    assert list(sorted(l))[-1] == max(l)


@qc
def sort_preserves_length(l=list_(int, max_length=64)):
    yield classify(len(l) == 0, "empty list")
    yield classify(len(l) % 2 == 0, "even list")
    yield classify(len(l) % 2 != 0, "odd list")
    assert len(list(sorted(l))) == len(l)



"""
Integration between pyqcy and other testing libraries,
especially test runners (such as unittest or nose).
"""
import unittest

from .properties import Property
from .runner import run_tests


__all__ = ['TestCase']


class TestCase(unittest.TestCase):
    """Test case containing pyqcy properties that should be tested
    automatically, as part of a standard unit test run.
    Properties should be defined directly within a subclass,
    using a standard @qc decorator - for example:

        class Sorting(TestCase):
            '''Properties that must hold for a sorting.'''
            @qc
            def sort_preserves_length(l=list_(of=int, max_length=128)):
                assert len(l) == len(list(sorted(l)))
            @qc
            def sort_finds_minimum(
                l=list_(of=int, min_length=1, max_length=128)
            ):
                assert min(l) == list(sorted(l))[0]

    Since pyqcy.TestCase is a subclass of standard unittest.TestCase,
    it will be discovered by unittest.main(), nosetests and similar
    testing utilities.
    """
    def test(self):
        """Runs tests for all properties defined within this class."""
        props = [v for v in self.__class__.__dict__.itervalues()
                 if isinstance(v, Property)]
        run_tests(props)



"""
Unit tests for integration facilities.
"""
import unittest
from pyqcy import *

import os


class Integration(unittest.TestCase):
    """Test cases for the pyqcy test case. (Yes.)"""

    def test_testcase(self):
        """The test case for TestCase."""
        class Sorting(TestCase):
            @qc
            def sort_preserves_length(l=list_(of=int, max_length=128)):
                assert len(l) == len(list(sorted(l)))

            @qc
            def sort_finds_minimum(
                l=list_(of=int, min_length=1, max_length=128)
            ):
                assert min(l) == list(sorted(l))[0]

        suite = unittest.TestLoader().loadTestsFromTestCase(Sorting)
        results = unittest.TextTestRunner(
            stream=open(os.devnull, 'w'),  # we don't want output here
            verbosity=0).run(suite)

        assert results.wasSuccessful()



"""
Simple, built-in test runner.
"""
import sys
import os
import traceback

from .properties import Property, DEFAULT_TEST_COUNT
from .utils import partition


__all__ = ['main']


def main(module='__main__', exit=True, failfast=False):
    """Test runner. When called, it will look for all properties
    (i.e. functions with @qc decorator) and push them through
    a default number of checks.
    Arguments are intended to mimic those from unittest.main().
    Return value is the total number of properties checked
    (provided exit=False and program doesn't terminate).
    """
    if isinstance(module, basestring):
        module_name = module
        module = __import__(module_name)
        for part in module_name.split('.')[1:]:
            module = getattr(module, part)

    props = [v for v in module.__dict__.itervalues()
             if isinstance(v, Property)]

    success = run_tests(props, failfast=failfast)
    if exit:
        sys.exit(0 if success else 1)
    return len(props)


def run_tests(props, failfast=False):
    """Executes tests for given list of properties.
    Returns boolean flag indicating if all the tests succeeded.
    """
    success = True

    for p in props:
        try:
            results = p.test()
        except:
            print "%s: failed." % p.func.__name__
            traceback.print_exception(*sys.exc_info())
            success = False
            if failfast:
                break
        else:
            print_test_results(p, results)

    return success


def print_test_results(prop, results):
    """Prints results of testing a single property.
    Results include any statistical information that the property
    has generated though `yield` statements.
    """
    print "%s: passed %s test%s." % (prop.func.__name__,
        DEFAULT_TEST_COUNT,
        "s" if DEFAULT_TEST_COUNT != 1 else "")

    # gather and display statistics
    with_stats, without_stats = partition(lambda r: len(r) > 0,
                                          results)
    if len(with_stats) > 0:
        stats = {}
        for s in with_stats:
            stats[s] = stats.get(s, 0) + 1
        stats = stats.items()
        if len(without_stats) > 0:
            stats.append(("<rest>", len(without_stats)))

        results_count = float(len(results))
        for labels, count in stats:
            percentage = "%.2f%%" % (count * 100 / results_count)
            summary = ", ".join(map(str, labels))
            print "%s: %s" % (percentage.rjust(5), summary)



"""
Arbitrary values generators
for standard Python types.
"""
import sys
import random
import functools

from pyqcy.arbitraries import arbitrary, is_arbitrary


# Arbitrary values' generators for built-in scalar types

@arbitrary(int)
def int_(min=-sys.maxint - 1, max=sys.maxint):
    """Default arbitrary values' generator for the int type."""
    return random.randint(min, max)


@arbitrary(float)
def float_(min=-float(sys.maxint), max=float(sys.maxint)):
    """Default arbitrary values' generator for the float type."""
    return min + random.random() * (max - min)


@arbitrary(complex)
def complex_(min_real=-float(sys.maxint), max_real=float(sys.maxint),
             min_imag=-float(sys.maxint), max_imag=float(sys.maxint)):
    """Default arbitrary values' generator for the complex type."""
    reals = float_(min_real, max_real)
    imags = float_(min_imag, max_imag)
    return complex(next(reals), next(imags))


@arbitrary(str)
def str_(of=int_(min=0, max=255), min_length=1, max_length=64):
    """Default arbitrary values' generator for strings."""
    length = random.randint(min_length, max_length)
    if is_arbitrary(of):
        return ''.join(chr(next(of)) for _ in xrange(length))
    return ''.join(random.choice(of) for _ in xrange(length))


# Arbitrary values' generators for built-in collection types

@arbitrary
def tuple_(*args, **kwargs):
    """Generator for arbitrary tuples. Resulting tuples are always
    of same length, equal to number of arbitrary generators passed
    to this function or the value of `n` keyword argument.
    """
    n = kwargs.get('n')
    if n is None:
        return tuple(map(next, args))

    of = kwargs.get('of')
    if of:
        if args:
            raise TypeError("ambiguous invocation - "
                            "more than one possible type for tuple elements")
    else:
        if len(args) != 1:
            raise TypeError("no/invalid type of arbitrary tuple elements")
        of = args[0]

    return tuple(next(of) for _ in xrange(n))


two = functools.partial(tuple_, n=2)
three = functools.partial(tuple_, n=3)
four = functools.partial(tuple_, n=4)


@arbitrary
def list_(of, min_length=0, max_length=1024):
    """Generator for arbitrary lists. List elements themselves
    can come from any arbitrary generator passed as first argument.
    """
    length = random.randint(min_length, max_length)
    if is_arbitrary(of):
        return [next(of) for _ in xrange(length)]
    return [random.choice(of) for _ in xrange(length)]


@arbitrary
def dict_(keys=None, values=None, items=None,
          min_length=0, max_length=1024):
    """Generator for arbitrary dictionaries. Either `keys` and `values`,
    or the `items` argument must be provided - but not both.
    """
    kv_provided = keys is not None and values is not None
    items_provided = items is not None
    if not (kv_provided or items_provided):
        raise ValueError("no generators for dictionary items provided")
    if kv_provided and items_provided:
        raise ValueError("ambiguous invocation - "
                         "provide either keys and values, or items")

    next_item = ((lambda: next(items)) if items_provided else
                 (lambda: (next(keys), next(values))))
    length = random.randint(min_length, max_length)
    return dict(next_item() for _ in xrange(length))



"""
Generators of arbitrary values.
"""
import functools
import inspect

from pyqcy.utils import optional_args


@optional_args
class arbitrary(object):
    """@arbitrary decorator which can be applied on functions
    that are intended to output "random" (arbitrary) values
    of given type.
    """
    # Dictionary mapping types into
    # generators of arbitrary values for those types
    registry = {}

    def __init__(self, type_=None):
        self.type_ = type_

    def __call__(self, func):
        """Applies the @arbitrary decorator to given function.
        If `type` argument was supplied previously,
        the resulting generator will be remembered in global registry
        for easy reference.
        """
        if self.type_ is None:
            gen_func = self.__arbitrary_generator(func)
        else:
            def validate(value):
                if not isinstance(value, self.type_):
                    raise TypeError(
                        "arbitrary value %r is of type %s; expected %s" % (
                            value, type(value).__name__, self.type_.__name__))
                return value

            gen_func = self.__arbitrary_generator(func, validate)
            self.registry.setdefault(self.type_, [])
            self.registry[self.type_].append(gen_func)

        gen_func._arbitrary = True  # marker attribute
        return gen_func

    def __arbitrary_generator(self, func, value_func=None):
        """Constructs arbitrary generator based on given function `func`.
        It can be both a function that returns a single value,
        or a generator function.
        """
        if value_func is None:
            value_func = lambda v: v  # identity function

        if inspect.isgeneratorfunction(func):
            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                args, kwargs = self.__coerce_to_arbitraries(args, kwargs)
                for obj in func(*args, **kwargs):
                    yield value_func(obj)
        else:
            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                args, kwargs = self.__coerce_to_arbitraries(args, kwargs)
                while True:
                    yield value_func(func(*args, **kwargs))

        return wrapper

    def __coerce_to_arbitraries(self, args=[], kwargs={}):
        """Ensures given list and dictionary of positional and keyword
        arguments contains appropriate arbitrary values' generators.
        Elements that cannot be reasonably coerced into arbitraries
        are left unchanged.
        """
        args = [to_arbitrary(arg) if is_arbitrary(arg) else arg
                for arg in args]
        kwargs = dict((k, to_arbitrary(v) if is_arbitrary(v) else v)
                      for (k, v) in kwargs.iteritems())
        return args, kwargs


def is_arbitrary(obj):
    """Checks whether given object can work as generator of arbitrary values.
    This functions handles all the forms in which arbitraries can occur
    in the code, including: generators, generator functions, and types.
    """
    if inspect.isgenerator(obj):
        return True
    if inspect.isgeneratorfunction(obj):
        return getattr(obj, '_arbitrary', False)
    if isinstance(obj, type):
        return obj in arbitrary.registry
    return False


def to_arbitrary(obj):
    """Ensures that given object is a generator of arbitrary values.
    Rather than permitting only actual generators, this allows
    us to pass generator functions or even types, provided
    there is a known arbitrary generator for them.
    """
    if inspect.isgenerator(obj):
        return obj
    if inspect.isgeneratorfunction(obj):
        return obj()    # fails if arguments are required,
                        # and this is intended

    # looking up types in global registry
    if isinstance(obj, type):
        arbit_gens = arbitrary.registry.get(obj)
        if not arbit_gens:
            raise TypeError(
                "no arbitrary values' generator found for type: %s" % obj)
        return to_arbitrary(arbit_gens[0])

    raise ValueError(
        "invalid generator of arbitrary values: %r (of type %s)" % (
            obj, type(obj).__name__))


from .standard import *
from .combinators import *



"""
Unit tests for the pyqcy test runner.
"""
import unittest
from pyqcy import *


class Runner(unittest.TestCase):
    """Test cases for the pyqcy test runner (pyqcy.main)."""

    def test_runner(self):
        """Tests the runner. This uses all the @qc properties
        from all test modules imported by nosetests, and runs
        them under our test harness to verify it functions
        properly.
        """
        from pyqcy.properties import Property
        assert main(exit=False) == len([obj for obj in globals().itervalues()
                                        if isinstance(obj, Property)])



"""
Unit tests for arbitraries.
"""
import unittest
from pyqcy import *

import random
import json


class Arbitraries(unittest.TestCase):
    """Test cases for different types of @arbitrary generators."""

    def test_standard_arbitrary(self):
        addition_doesnt_break.test()

    def test_generator_arbitrary(self):
        multiplication_on_floats.test()

    def test_standard_arbitrary_with_args(self):
        addition_works_for_positive_floats.test()

    def test_standard_nested_arbitrary(self):
        sort_finds_minimum.test()

    def test_tuple_arbitrary(self):
        tuples_survive_listification.test()

    def test_shorthand_tuple_arbitrary(self):
        two_is_two.test()

    def test_dict_arbitrary(self):
        dict_update_works.test()

    def test_apply(self):
        apply_works_with_functions.test()
        apply_works_with_types.test()

    def test_data(self):
        data_works_with_lists.test()
        data_works_with_tuples.test()
        data_works_with_dictionaries.test()

    def test_elements_arbitrary(self):
        set_of_elements.test()

    def test_one_of_arbitrary(self):
        one_of_works.test()

    def test_frequency_arbitrary(self):
        frequency_works.test()

    def test_custom_arbitrary(self):
        case_transform_preserves_length.test()


# Arbitraries

@arbitrary(str)
def str_with_len_lt5():
    length = random.randint(0, 4)
    return ''.join(chr(random.randint(0, 255))
                   for _ in xrange(length))


@arbitrary(float)
def random_floats():
    while True:
        yield random.random()


# Test properties

@qc
def addition_doesnt_break(x=int, y=int):
    _ = x + y


@qc
def addition_works_for_positive_floats(
    x=float_(min=0.0), y=float_(min=0.0)
):
    sum = x + y
    assert sum >= x and sum >= y


@qc
def case_transform_preserves_length(
    s=str_with_len_lt5
):
    assert len(s) == len(s.upper())
    assert len(s) == len(s.lower())


@qc
def multiplication_on_floats(
    a=random_floats, b=random_floats
):
    assert a * b <= 1.0


@qc
def sort_finds_minimum(
    l=list_(of=int, min_length=1, max_length=64)
):
    assert sorted(l)[0] == min(l)


@qc
def dict_update_works(
    d1=dict_(keys=str, values=str, min_length=1, max_length=64),
    d2=dict_(items=tuple_(int, str), min_length=1, max_length=64)
):
    d1_len = len(d1)
    d1.update(d2)
    assert len(d1) == d1_len + len(d2)


@qc
def tuples_survive_listification(
    t=tuple_(int, n=10)
):
    as_list = list(t)
    back_as_tuple = tuple(as_list)
    assert all((old, new) for (old, new) in zip(t, back_as_tuple))


@qc
def two_is_two(
    t=two(int)
):
    assert len(t) == 2


@qc
def apply_works_with_functions(
    x=apply(json.dumps,
            list_(of=int, min_length=1, max_length=64))
):
    assert isinstance(x, basestring)
    assert all(isinstance(i, int) for i in json.loads(x))


@qc
def apply_works_with_types(
    x=apply(str, int)
):
    assert isinstance(x, str)


@qc
def data_works_with_lists(
    x=data([int, str])
):
    assert isinstance(x, list)
    assert len(x) == 2
    assert isinstance(x[0], int) and isinstance(x[1], str)


@qc
def data_works_with_tuples(
    x=data((float, str, int))
):
    assert isinstance(x, tuple)
    assert len(x) == 3
    for i, t in enumerate((float, str, int)):
        assert isinstance(x[i], t)


@qc
def data_works_with_dictionaries(
    x=data({
        'login': str_(of='abcdefghijklmnopqrstuvwxyz',
                      min_length=4, max_length=16),
        'password': str_(min_length=8, max_length=64),
    })
):
    assert isinstance(x, dict)
    assert len(x) == 2
    assert 'login' in x and 'password' in x
    assert isinstance(x['login'], str) and isinstance(x['password'], str)


@qc
def set_of_elements(
    x=elements("abc")
):
    assert len(x) == 1
    assert x in "abc"


@qc
def one_of_works(
    x=one_of(float, int)
):
    assert isinstance(x, (float, int))


@qc
def frequency_works(
    x=frequency([(1, float), (2, int)])
):
    assert isinstance(x, (float, int))



#!/usr/bin/env python
"""
pyqcy
=====

*pyqcy* [pyksi:] is an automated testing framework for Python
inspired by the wonderful *QuickCheck* library for Haskell.

What's the idea?
----------------

Instead of writing fully-fledged test cases, with *pyqcy*
you simply define logical **properties** that your code
is expected to satisfy. Based on that, *pyqcy* will
automatically generate test cases for you - hundreds of them,
in fact!

How does it look like?
----------------------

For starters, try this:

::

    from pyqcy import qc, int_, main

    @qc
    def addition_actually_works(
        x=int_(min=0), y=int_(min=0)
    ):
        the_sum = x + y
        assert the_sum >= x and the_sum >= y

    if __name__ == '__main__':
        main()

::

    $ pip install pyqcy
    $ python test.py
    addition_actually_works: passed 100 tests.

*pyqcy* generated 100 test cases and checked whether
they all pass. For a few lines of code, that's rather nice,
isn't it? :)

Of course, you are not limited to integers - there is
built-in support for all standard types, including lists
and dictionaries. Custom classes can be used as well:
just specify how to generate an *@arbitrary* object
of your class, and you're set.

Hey, I like it!
---------------

Then check these links to find out more:

* `website <http://xion.github.com/pyqcy>`_
* `github <http://github.com/Xion/pyqcy>`_
* `development version
    <http://github.com/Xion/pyqcy/zipball/master#egg=pyqcy>`_

"""
from setuptools import setup, find_packages


setup(
    name="pyqcy",
    version="0.3.1",
    description="QuickCheck-like testing framework for Python",
    long_description=__doc__,
    author='Karol Kuczmarski "Xion"',
    author_email="karol.kuczmarski@gmail.com",
    url="http://xion.github.com/pyqcy",
    license="Simplified BSD",

    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
    ],

    platforms='any',
    packages=find_packages(),
    tests_require=['nose'],
)



"""
Statistics for test cases.
"""
import collections


__all__ = ['collect', 'classify']


class Tag(object):
    """A tag that can be assigned to a test case.
    It can be any hashable value, i.e. a one that
    could be part of a set, or a key in dictionary.

    This is mostly a marker class - a wrapper that
    is used to distinguish results of statistic function
    calls from other values that could be yielded
    from property function.
    """
    def __init__(self, value):
        if not isinstance(value, collections.Hashable):
            raise TypeError("tag value must be hashable")
        self.value = value


def collect(value):
    """Collects test cases that share the same value
    (passed as argument) for statistical purposes.
    The value can be any hashable.
    Typical usage is as follows:

        @qc
        def sort_works(
            l=list_(int, min_length=1, max_length=64)
        ):
            yield collect(len(l))
            assert list(sorted(l))[0] = min(l)
    """
    return Tag(value)


def classify(condition, label):
    """Classifies test cases depending on whether they satisfy
    given condition. If a test case meets the condition, it will
    "stamped" with given label that will subsequently appear
    in statistical report.
    Typical usage is as follows:

        @qc
        def sort_preserves_length(l=list_(int, max_length=64)):
            yield classify(len(l) == 0, "empty list")
            yield classify(len(l) < 10, "short list")
            assert len(list(sorted(l))) == len(l)
    """
    return Tag(label) if condition else None



"""
Combinators for generators of arbitrary values.
"""
import functools
import inspect
import random

from collections import Iterable, Mapping
try:
    from collections import OrderedDict
except ImportError:
    OrderedDict = dict # fallback for Python 2.6

from pyqcy.arbitraries import arbitrary, is_arbitrary, to_arbitrary
from pyqcy.utils import recursive


def apply(func, *args, **kwargs):
    """Creates a generator that applies a function to objects
    returned by given generator(s). Arguments to that function
    will be passed in the same manner as the generators have
    been passed to apply(), i.e. positional args will be passed
    as positional and keyword as keyword.
    """
    if not func:
        raise ValueError("no function provided")
    if not callable(func):
        raise TypeError("expected a callable")

    func_args = [next(to_arbitrary(arg)) for arg in args]
    func_kwargs = dict((k, next(to_arbitrary(v)))
                       for (k, v) in kwargs.iteritems())

    @arbitrary
    def generator():
        return func(*func_args, **func_kwargs)
    return generator


def data(schema):
    """Creates a generator which outputs data structures conforming
    to given schema. Schema can be a list or dictionary that contains
    either immediate values or other arbitrary generators.
    """
    if schema is None:
        raise ValueError("no schema specified")

    is_data_structure = (isinstance(schema, Iterable)
                         and not inspect.isgenerator(schema))
    if not is_data_structure:
        raise TypeError("schema must be a data structure")

    def instance_of(s):
        """Constructs a new data structure instance conforming
        to given schema. This function proceeds recursively.
        """
        if isinstance(s, Mapping):
            res = (OrderedDict()
                   if isinstance(s, OrderedDict)
                   else dict())
            items = s.iteritems()
        else:
            res = [None] * len(s)
            items = enumerate(s)

        for key, value in items:
            if is_arbitrary(value):
                value = to_arbitrary(value)
                res[key] = next(value)
            elif isinstance(value, Iterable):
                res[key] = instance_of(value)
            else:
                res[key] = value

        if isinstance(s, tuple):
            res = tuple(res)
        return res

    @arbitrary
    def generator():
        return instance_of(schema)
    return generator


def combinator(func):
    """Decorator for arbitrary combinator functions which take
    a collection of arguments as either an actual list/sequence,
    or as positional arguments. In other words, it makes
    it possible to use the following two forms of invocation:
    >>> func([1, 2, 3])
    >>> func(1, 2, 3)
    In both cases func() receives 1, 2 and 3 as
    positional arguments (*args).
    """
    _2arbitrary = recursive(lambda obj: to_arbitrary(obj)
                                        if is_arbitrary(obj) else obj)

    @arbitrary
    @functools.wraps(func)
    def wrapped(*args, **kwargs):
        if not args:
            return func(**kwargs)

        new_args = []
        for arg in args:
            arg_collection = (isinstance(arg, Iterable)
                              and not is_arbitrary(arg))
            if arg_collection:
                arg = map(_2arbitrary, arg)
                new_args.extend(arg)
            else:
                arg = _2arbitrary(arg)
                new_args.append(arg)
        return func(*new_args, **kwargs)

    return wrapped


@combinator
def elements(*args):
    """Creates a generator that returns a random element from those given.
    Every element has equal probability of being picked.
    """
    if not args:
        raise ValueError("cannot pick random element from empty sequence")
    return random.choice(args)


@combinator
def one_of(*args):
    """Creates a generator that chooses among given generators, giving
    equal probability to each one.
    """
    if not args:
        raise ValueError("no generators to choose from")
    return next(random.choice(args))


@combinator
def frequency(*args):
    """Creates a generator that chooses among given generators, according
    to probability (frequency) assigned to them. Function accepts pairs
    (or list of pairs) where the first element is the probability
    and the second element is a particular generator.
    """
    if not args:
        raise ValueError("no generators to choose from")

    freq_sum = sum((p for p, _ in args), 0)
    i = int(random.random() * freq_sum)

    s = 0
    for p, gen in args:
        if s <= i < s + p:
            return next(gen)
        s += p
